/**
 * For copyright see license.txt.
 */
package anasy.input;


/**
 * Class to represent input text to be parsed.
 * @author jure
 */
public class Input {
	
	public static final char NOCHAR = (char)-1;

	// current position
	protected int pos;
	// curent row
	protected int row;
	// current column
	protected int col;
	// input buffer
	protected String buf;

	public Input(String buf) {
		reset(buf);
	}

	public Input() {
		this("");
	}

	public int getPos() {
		return pos;
	}

	public int getRow() {
		return row;
	}

	public int getCol() {
		return col;
	}

	public Location getLoc() {
		return new Location(pos, row, col);
	}

	public String getBuf() {
		return buf;
	}

	public void reset(String str) {
		buf = str;
		pos = 0;
		row = col = 1;
	}

	public void append(String str) {
		if (str != null) buf += str;
	}

	public int available() {
		return buf.length() - pos;
	}

	public boolean ready() {
		return available() > 0;
	}

	public char peek(int ahead) {
		if (available() > ahead) return buf.charAt(pos + ahead);
		return NOCHAR;
	}

	public char peek() {
		return peek(0);
	}

	public boolean peek(char ch) {
		return peek() == ch;
	}
	
	public char advance() {
		char ch = peek();
		pos++;
		if (ch == '\n') {
			row++;
			col = 1;
		} else if (ch == '\t')
			col = ((col - 1) / 4) * 4 + 5;
		else
			col++;
		return ch;
	}

	public int advance(int cnt) {
		int c = 0;
		while (available() > 0 && c++ < cnt) advance();
		return c;
	}
	
	public boolean advanceIf(char ch) {
		if (peek() != ch) return false;
		advance();
		return true;
	}

/*	public char advance(char ch) throws SyntaxError {
		if (peek() != ch) return advance();
		throw new SyntaxError(String.format("Character '%c' expected", ch), getLoc());		
	}*/

	public boolean advanceIf(String str) {
		for (int i = 0; i < str.length(); i++)
			if (peek(i) != str.charAt(i)) return false;
		advance(str.length());
		return true;
	}

/*	public void advance(String str) throws SyntaxError {
		if (!advanceIf(str))
			throw new SyntaxError(String.format("String '%s' expected", str), getLoc());
	}*/

	public boolean advanceDigits(int radix) {
		boolean moved = false;
		while (Character.digit(peek(), radix) != -1) {
			advance();
			moved = true;
		}
		return moved;
	}

	public String extract(Location loc, int len) {
		int p = (len < 0) ? pos + len : loc.pos + len; 
		return buf.substring(loc.pos, p);	
	}

	public String extract(Location loc) {
		return extract(loc, pos - loc.pos);
	}

}
