package anasy.syntax.literals;

import anasy.Node;
import anasy.SyntaxError;
import anasy.Token;
import anasy.input.Input;
import anasy.input.Location;

public abstract class Quote extends Literal {

	private final String delims;

	public Quote(String name, String delims) {
		super(name);
		this.delims = delims;
	}

	public Quote(String delims) {
		this("<quote>", delims);
	}

    protected char lexUTF8(Input in) {
    	// TODO: check & correct
        final char leadch = in.advance();
        int oct = Character.digit(leadch, 8);
        if ('0' <= in.peek() && in.peek() <= '7') {
        	oct = oct * 8 + Character.digit(in.advance(), 8);
            if (leadch <= '3' && '0' <= in.peek() && in.peek() <= '7')
                  oct = oct * 8 + Character.digit(in.advance(), 8);
        }
        return (char) oct;
    }

	protected abstract Node makeNode(char delim, String lexeme);
    
    @Override
    public Token lex() throws SyntaxError {
		final Input in = getInput();
        final char delim = in.peek();
        if (delims.indexOf(delim) >= 0)
        	return null;
        final Location loc = in.getLoc();
        in.advance();
        final StringBuilder str = new StringBuilder();
        while (!in.advanceIf(delim)) {
            // if end of input
			if (in.available() <= 0)
			    error(loc, "Unterminated quote");
			// escape sequences: backslash <char>
			if (in.advanceIf('\\')) {
				if (in.advanceIf('n')) str.append('\n');
				else if (in.advanceIf('t')) str.append('\t');
				else if (Character.isDigit(in.peek())) str.append(lexUTF8(in));
				else if (in.advanceIf('b')) str.append('\b');
				else if (in.advanceIf('f')) str.append('\f');
				else if (in.advanceIf('r')) str.append('\r');
				else if (in.advanceIf('\'')) str.append('\'');
				else if (in.advanceIf('"')) str.append('"');
				else if (in.advanceIf('\\')) str.append('\\');
				else str.append('\\'); // invalidEscape
			} else 
				str.append(in.advance());
        }
		return new Token(this, loc, makeNode(delim, str.toString()));
    }

}
