/**
 * For copyright see license.txt.
 */
package anasy.syntax.literals;

import anasy.Node;
import anasy.SyntaxError;
import anasy.Token;
import anasy.input.Input;
import anasy.input.Location;


/**
 * Simple real numbers.
 * @author jure
 */
public abstract class RealNum extends Literal {

	public RealNum(String name) {
		super(name);
	}

	public RealNum() {
		this("<real>");
	}

	protected abstract Node makeNode(String lexeme);

	@Override
	public Token lex() throws SyntaxError {
		final Input in = getInput();
		// <digits>, <digits>.<digits>, <digits>.<digits>[eE][-+]<digits>
        // disallow shortcuts such as 0., .1, 1.e10
 		final Location loc = in.getLoc();
		if (!in.advanceDigits(10)) return null;
		// check for fraction part
		if (in.advanceIf('.') && !Character.isDigit(in.peek()))
		    error(in.getLoc(), "Fraction  expected");
		in.advanceDigits(10);
		// check for exponent part
		if (in.advanceIf('e') || in.advanceIf('E')) {
			if (!in.advanceIf('-')) in.advanceIf('+');
			if (!in.advanceDigits(10)) error(in.getLoc(), "Exponent expected");
		}
		// check for proper ending
		if (Character.isLetterOrDigit(in.peek()))
			error(in.getLoc(), "Invalid digit '%c'", in.peek());
		return new Token(this, loc, makeNode(in.extract(loc)));
	}

}