/**
 * For copyright see license.txt.
 */
package anasy.syntax.operators;

import anasy.Node;
import anasy.SyntaxError;
import anasy.Token;
import anasy.Tokind;
import anasy.input.Lexer;
import anasy.input.Location;


/**
 * Outfix operators, e.g., parentheses, brackets, ...
 * @author jure
 *
 * @param <N>
 */
public class Outfix extends Operator {

	public final Operator tokClose;

	public Outfix(String lexemeOpen, String lexemeClose, int lbp) {
		super(lexemeOpen, lbp);
		tokClose = new Operator(lexemeClose, 0);
	}

	@Override
	public void setParent(Tokind parent) {
		super.setParent(parent);
		parent.add(tokClose);
	}

	protected Node makeNode(Node operand) {
		return operand;
	}

	@Override
	public Token parse(int rbp) throws SyntaxError {
		final Lexer lexer = getLexer();
		final Location loc = lexer.prev().loc;
		// check if open token is immediately followed by close token
		Token token = lexer.peek(tokClose) ? null : parent.parse();
		// advance newline
//			parser.advanceIf(parser.tokenNewline);
		// advance the closing token
		lexer.advance(tokClose);
		// return new object generated from content
		return new Token(parent, loc, makeNode(token.value));
	}

}