/**
 * For copyright see license.txt.
 */
package anasy.syntax.operators;

import anasy.Node;
import anasy.SyntaxError;
import anasy.Token;
import anasy.input.Lexer;


/**
 * Prefix and infix operators.
 * @author jure
 *
 * @param <N>
 */
public abstract class PrefixInfix extends Operator {

	public final int pbp;				// prefix bounding precedence
	public final boolean rightAssoc;	// associativity for infix

	public PrefixInfix(String name, int pbp, int lbp, boolean rightAssoc) {
		super(name, lbp);
		this.pbp = pbp;
		this.rightAssoc = rightAssoc;
	}

	public PrefixInfix(String name, int pbp, int lbp) {
		this(name, pbp, lbp, false);
	}

	public abstract Node makePrefixNode(Node operand);
	
	public abstract Node makeInfixNode(Node left, Node right);
	
	@Override
	public Token parse(int rbp) throws SyntaxError {
		final Lexer lexer = getLexer();
		final Node right = parent.parse(pbp).value;
		return new Token(parent, lexer.prev().loc, makePrefixNode(right));
	}

	@Override
	public Token parse(Token left, int rbp) throws SyntaxError {
		final Lexer lexer = getLexer();
		final Token right = parent.parse(rightAssoc ? lbp - 1: lbp);
		return new Token(parent, lexer.prev().loc, makeInfixNode(left.value, right.value));
	}

}