package anasy.syntax.special;

import anasy.SyntaxError;
import anasy.Token;
import anasy.Tokind;
import anasy.input.Input;
import anasy.input.Lexer;
import anasy.syntax.operators.Operator;


public class Start extends Tokind {

	public final Tokind tokBegin = new Operator("<begin>", 0);
    public final Tokind tokEnd = new Operator("<end>", 0);
    public final Tokind tokInvalid = new Operator("<invalid>", 0);
    public final Tokind tokMain;

	protected final Input input;
	protected final Lexer lexer = new Lexer(this);

	public Start(Input input, Tokind main) {
		super("<start>", 0);
		this.input = input;
		this.tokMain = main;
		main.setParent(this);
	}

	@Override
	protected Input getInput() {
		return input;
	};

	@Override
	protected Lexer getLexer() {
		return lexer;
	}

	@Override
	public void ensure() throws SyntaxError {}

	@Override
	public Token lex() throws SyntaxError {
		Token token;
		token = tokMain.lex();
		if (token == null)
			token = new Token(input.available() > 0 ? tokInvalid : tokEnd, input.getLoc());
		return token;
	}
	
	public Token parse(int rbp) throws SyntaxError {
	    lexer.begin(new Token(tokBegin, input.getLoc()));
	    ensure();
 	    if (lexer.peek(tokEnd)) return null;	// !!! empty input
	    Token r = tokMain.parse(rbp);
	    if (!lexer.advanceIf(tokEnd))
	        error(lexer.peek().loc, "End of input expected, but '%s' found", lexer.peek());
	    return r;
	}

}
