/**
 * For copyright see license.txt.
 */
package anasy.syntax.whitespace;

import java.util.Stack;

import anasy.SyntaxError;
import anasy.Token;
import anasy.Tokind;
import anasy.input.Input;
import anasy.input.Location;
import anasy.syntax.operators.Operator;


/**
 * Indenter generats Newline, Indent, and Dedent tokens.
 * @author jure
 *
 * @param <N>
 */
public class Indenter extends Tokind {

	public final Tokind tokNewline = new Operator("<newline>", 0);
	public final Tokind tokIndent = new Operator("<indent>", 0);
	public final Tokind tokDedent = new Operator("<dedent>", 0);
	// indentation levels stack
	private Stack<Integer> indents;
	//
	private int dedentCount;

	public Indenter(String name) {
		super(name, 0);
		indents = new Stack<Integer>();
		indents.clear();
		indents.add(1);
		dedentCount = 0;
	}

	public Indenter() {
		this("<indenter>");
	}
	
	private void checkIndentation(Input in) throws SyntaxError {
		// count dedentations
		while (in.getCol() < indents.peek()) {
			indents.pop();
			dedentCount++;
		}
		// check indentation level
		if (in.getCol() != indents.peek())
			error(in.getLoc(), "invalid indentation level");
	}

	@Override
	public Token lex() throws SyntaxError {
		final Input in = getInput();
		// generate any Dedents
		if (dedentCount > 0) {
			dedentCount--;
			return new Token(tokDedent, in.getLoc());
		}
        if (!Character.isWhitespace(in.peek())) return null;
		// after Dedents generate also Newline (if not end of input)
//			if (dedented && lexer.peek() > 0)
//				tokens.add(new Token(tokenNewline, lexer.getLoc()));
        Location loc = in.getLoc();
		Location locnl = null;	// location of newline
		while (Character.isWhitespace(in.peek())) {
			if (in.peek() == '\n') locnl = in.getLoc();	// peek() is before
			in.advance();								// advance(): locnl is at end of line 
		}
		// if no newline then return whitespace
		if (locnl == null)
			return new Token(this, loc);
		// newline present, check columns
		int col = indents.peek();
		if (in.getCol() == col) {
			if (in.available() <= 0) return null;
			return new Token(tokNewline, locnl);
		}
		// in.col > col => indentation
		if (in.getCol() > col) {
			indents.push(in.getCol());
			return new Token(tokIndent, locnl);
		}
		// otherwise: in.col < col => dedentation
		checkIndentation(in);
		dedentCount--;		// dedentCount>0
		return new Token(tokDedent, in.getLoc());
	}

	@Override
	public boolean activate() {
		return false;
	}
}